//
//  ES1Renderer.m
//  hw2_graphics
//
//  Created by slork on 1/12/10.
//  Copyright Stanford University 2010. All rights reserved.
//

#import "ES1Renderer.h"
#import "mo_audio.h"
#import "mo_touch.h"
#import "mo_accel.h"
#import "mo_gfx.h"
#import "Stk.h"
#import <math.h>
#import "SSObject.h"


#define SRATE		24000
#define FRAMESIZE	512
#define NUMCHANNELS	2


#define X_RESOL				480
#define Y_RESOL				320

#define XAXIS_MAPPING(X)	(((X_RESOL/2) - (X))/(X_RESOL/3))
#define YAXIS_MAPPING(Y)	(((Y_RESOL/2) - (Y))/(Y_RESOL/2))

#define MAX_PROJECTILE		10
#define MAX_SOUND_CLIP		10

SSRubberBand g_rubberBand;
std::vector<SSProjectile*> g_projectile;

UITouch *g_1stTouch=NULL;
UITouch *g_2ndTouch=NULL;
UITouch *g_3rdTouch=NULL;
int g_1stTouchKey=0;
int g_2ndTouchKey=0;
int g_3rdTouchKey=0;

int g_touchedBullet=-1;

Vector3D g_gravity(0.f, 0.f, 0.f);

GLuint g_texture[21];
NSString *explosionName[16] = {
	@"explosion0",	@"explosion1",
	@"explosion2",	@"explosion3",
	@"explosion4",	@"explosion5",
	@"explosion6",	@"explosion7",
	@"explosion8",	@"explosion9",
	@"explosion10",	@"explosion11",
	@"explosion12",	@"explosion13",
	@"explosion14",	@"explosion15"
};


// controllable parameters
Float32 g_tension = 0.8f;
Float32 g_wallResist = 0.95f;
Float32 g_airResist = 0.98f;
bool g_enableShakeClear=true;
bool g_enableGravity=true;
bool g_eraseBand=true;
Float32 g_reverbTime=0.f;
int g_themesIndex=0;




//---------------------------------------------------------------------
// name: newProjectile()
// description: Make new projectile
//---------------------------------------------------------------------
int newProjectile(CGPoint pos)
{
	Vector3D point(pos.x, pos.y, 0);
	
	g_projectile.push_back(new SSProjectile(point));
	
	return g_projectile.size()-1;
}


//---------------------------------------------------------------------
// name: selectBullet()
// description: Select one of projectiles by touching
//---------------------------------------------------------------------
int selectBullet(CGPoint pos)
{
	pos.x = MIN(MAX(pos.x, 0.f), Y_RESOL);
	pos.x = MIN(MAX(pos.x, 0.f), Y_RESOL);
	
	Vector3D newPos(XAXIS_MAPPING(pos.y), YAXIS_MAPPING(pos.x), 0);
	int index=0;
	
	for( std::vector<SSProjectile*>::iterator it = g_projectile.begin(); it != g_projectile.end(); it++ ){
		Vector3D point = (*it)->getPoint();
		GLfloat dist = (point - newPos).magnitude();
		
		if( dist < 0.2f ) {
			(*it)->stopProjectile();
			
			return index;
		}
		
		index++;
	}
	
	return -1;
}



//---------------------------------------------------------------------
// name: movingBullet()
// description: Moving a projectile by dragging
//---------------------------------------------------------------------
void movingBullet(int index, CGPoint pos)
{
	
	std::vector<SSProjectile*>::iterator it = g_projectile.begin();
	it += index;
	(*it)->setPoint(pos);
	
}




//---------------------------------------------------------------------
// name: collisionProc()
// description: Detect collision between projectiles and phisically model
//---------------------------------------------------------------------
void collisionProc(AudioData *data)
{
	for( std::vector<SSProjectile*>::iterator it=g_projectile.begin(); it != g_projectile.end(); it++ ){
		for( std::vector<SSProjectile*>::iterator ot=it+1; ot != g_projectile.end(); ot++ ){
			
			Vector3D diff = (*it)->getPoint() - (*ot)->getPoint();
			
			
			// if the distance b/w two projectiles is less than 2*radius of them,
			// collision happens
			if( diff.magnitude() < 2*PROJECTILE_RADIUS ) {
		
				// if one of projectiles is bullet of the star wars theme
				if( (*it)->getBullet() || (*ot)->getBullet() ) {
					if( (*it)->getBullet() && (*ot)->getBullet() ) {
						
						(*it)->removeProjectile();
						(*ot)->setExplosion();
						
					} else {
						
						// just explode a projectile which is not a bullet
						if( (*it)->getBullet() ) {
							(*it)->removeProjectile();
							(*ot)->setExplosion();
						} else {
							(*ot)->removeProjectile();
							(*it)->setExplosion();
						}
					}
					
					// play explosion sound
					data->eClip->startPlay();
					
					continue;
				}
				
				// find x-axis
				Vector3D xAxis = (*ot)->getPoint() - (*it)->getPoint();
				Vector3D U0x, U0y, U1x, U1y;
				GLfloat gain = xAxis.magnitude();
								
				xAxis *= 1.f/gain;
				
				// find projections
				GLfloat dot = xAxis * (*it)->getVelocity();
				U0x = xAxis * dot;
				U0y = (*it)->getVelocity() - U0x;
				
				Vector3D origin;
				dot = (origin-xAxis) * (*ot)->getVelocity();
				U1x = (origin-xAxis) * dot;
				U1y = (*ot)->getVelocity() - U1x;
				
				// find new velocity
				Vector3D V0x, V0y, V1x, V1y;
				V0x = U1x;
				V1x = U0x;
				
				// find the final velocity
				V0y = U0y;
				V1y = U1y;
				
				(*it)->setVelocity((V0x + V0y)*0.98f);	// 0.98 = loss by collision
				(*ot)->setVelocity((V1x + V1y)*0.98f);
				
				(*it)->setCollision();
				(*ot)->setCollision();
				
				GLfloat colPower = ((*it)->getVelocity() + (*ot)->getVelocity()).magnitude();
				
				if( colPower >= 0.3f ) {
					(*it)->setExplosion();
					(*ot)->setExplosion();
					
					// play explosion sound
					data->eClip->startPlay();					
				} else if( colPower >= 0.05f ) {
					
					// play collision sound
					data->cClip->startPlay();
				}
			}
		}
	}
	
	
}




//---------------------------------------------------------------------
// name: drawEllipse()
// description: Draw an ellipse
//---------------------------------------------------------------------

void drawEllipse (int segments, CGFloat width, CGFloat height, CGPoint center, bool filled)
{
	glPushMatrix();
	
	glTranslatef(center.x, center.y, 0);
	
	GLfloat vertices[segments*2];
	int count=0;
	
	for(int i=0; i < segments; i++){
		vertices[count++] = cos(TWO_PI*i/segments)*width;
		vertices[count++] = sin(TWO_PI*i/segments)*height;
	}
	
	glVertexPointer (2, GL_FLOAT , 0, vertices);
	glDrawArrays ((filled) ? GL_TRIANGLE_FAN : GL_LINE_LOOP, 0, segments);
	
	glPopMatrix();
}


//---------------------------------------------------------------------
// name: drawCircle()
// description: Draw an circle
//---------------------------------------------------------------------

void drawCircle (int circleSegments, CGFloat circleSize, CGPoint center, bool filled) 
{
	if( circleSize <= 0 )
		return;
	
	drawEllipse(circleSegments, circleSize, circleSize, center, filled);
}



//---------------------------------------------------------------------
// name: audioCallback()
// description: Audio callback function
//---------------------------------------------------------------------

void audioCallback(Float32 *buffer, UInt32 numFrames, void *userData)
{	
	AudioData *data = (AudioData *)userData;
	
	memset(buffer, 0, sizeof(SAMPLE)*numFrames*2);
	
	for(int i=0; i < numFrames; i++ ){
		SAMPLE sample = (data->pluck->tick()) + data->eClip->Tick()  + data->cClip->Tick();
		
		sample = data->reverb->tick(sample/4.f);
		
		buffer[2*i] = sample;
		buffer[2*i+1] = sample;
	}
	

	// if a rubber band is plucked, update a status of delay buffers and vertices of the rubber band
	g_rubberBand.updateStringState();
	
}


//---------------------------------------------------------------------
// name: touchCallback()
// description: Multi-touch callback function
//---------------------------------------------------------------------

void touchCallback( NSSet * touches, UIView * view, const std::map<int, MoTouchTrack> & touchPts, void * userData)
{	
	long count = [touches count];
	
	// when double taps occur, make new projectile
	if ( [[touches anyObject] tapCount] == 2 ) {
		static bool justOneTime = false;
		
		justOneTime = !justOneTime;
		
		if( justOneTime ) {
			UITouch *touch = [[touches allObjects] objectAtIndex:0];

			newProjectile([touch locationInView:view]);
		}
		return;
	}	
	
	
	// In case of a single touch, store a UITouch instance & see if
	//  1) one of projectiles is selected,
	//  2) one of projectiles is being dragged,
	//  3) a rubber band is being pulling
	if( count == 1 ) {

		int key = (touchPts.begin())->first;
		UITouch * touch = (touchPts.begin())->second.touch;

		if( [touch phase] == UITouchPhaseEnded || [touch phase] == UITouchPhaseCancelled ) {
			g_1stTouch = NULL;
			g_1stTouchKey = 0;
				
			if( g_eraseBand ) {
					
				// deselect projectile
				g_touchedBullet = -1;
				
			} else {

				if( g_rubberBand.isPulled() ) {
					// when selecting a projectile, shoot the selected one and deselect it
					if( g_touchedBullet >= 0 ) {
						std::vector<SSProjectile*>::iterator it=g_projectile.begin();
						it += g_touchedBullet;
					
						(*it)->setVelocity(g_rubberBand.getShotDirection()*0.1f);
						g_touchedBullet = -1;
					} else {
						newProjectile([touch locationInView:view]);

						g_touchedBullet = selectBullet([touch locationInView:view]);

						std::vector<SSProjectile*>::iterator it=g_projectile.begin();
						it += g_touchedBullet;
					
						if( g_themesIndex == 1 )
							(*it)->setBullet();	// only for star wars theme
						
						(*it)->setInitVelocity(g_rubberBand.getShotDirection()*.1f);
					}
				}
				
				// if the band is being pulled, start plucking
				g_rubberBand.startPlucking();
	
			}
				
		} else if( [touch phase] == UITouchPhaseBegan ){
				
			// add a touch as the 1st touch
			g_1stTouch = touch;
			g_1stTouchKey = key;
				
			// if one of projectiles is touched, select it
			g_touchedBullet = selectBullet([touch locationInView:view]);
			
			if( !g_eraseBand )
				g_rubberBand.checkPullingString([touch locationInView:view]);

		} else if([touch phase] == UITouchPhaseMoved ){

			// when one of projectiles is selected, move its position
			if( g_touchedBullet >= 0 )
				movingBullet(g_touchedBullet, [touch locationInView:view]);

			if( !g_eraseBand ) {
				g_rubberBand.checkPullingString([touch locationInView:view]);

				g_rubberBand.pullingString([touch locationInView:view]);
			}
		}
		
	// In case of double touches, update UITouch instances &
	// calculate initial vertices of a rubber band
	} else if (count == 2) {
		
		for (MoTouch::MoTouchMapIt it=touchPts.begin(); it!=touchPts.end(); it++) {    
			
			int key = it-> first;
			UITouch *touch = it->second.touch;
			
			if( [touch phase] == UITouchPhaseEnded || [touch phase] == UITouchPhaseCancelled ) {
				
				// update the 1st touch
				if( key == g_1stTouchKey ) {
					g_1stTouch = g_2ndTouch;
					g_1stTouchKey = g_2ndTouchKey;
					
					g_2ndTouch = NULL;
					g_2ndTouchKey = 0;
				} else {
					g_2ndTouch = NULL;
					g_2ndTouchKey = 0;
				}
				
				// remove a rubber band
				if( g_eraseBand )
					g_rubberBand.removeString();

			} else if( [touch phase] == UITouchPhaseBegan ) {
				
				// update the 1st & 2nd touches
				if( g_1stTouch == NULL ) {
					g_1stTouch = touch;
					g_1stTouchKey = key;
				} else if( key != g_1stTouchKey ) {
					g_2ndTouch = touch;
					g_2ndTouchKey = key;
				}
				
			}

		}

		// set a rubber band
		if( g_1stTouch != NULL && g_2ndTouch != NULL )
			g_rubberBand.setPoint([g_1stTouch locationInView:view], [g_2ndTouch locationInView:view]);
		
		
	// In case of triple touches, store a UITouch instance & 
	// pluck a rubber band,
	// make a projectile fly,
	// see if a projectile is selected, a projectile is being dragged and a rubber band is being pulling
	// If a rubber band is plucked, make sounds.
	} else if (count == 3) {
		
		for (MoTouch::MoTouchMapIt it=touchPts.begin(); it!=touchPts.end(); it++) {    
			
			int key = it-> first;
			UITouch *touch = it->second.touch;
			
			if( [touch phase] == UITouchPhaseEnded || [touch phase] == UITouchPhaseCancelled ) {
				
				// update the 1st, 2nd & 3rd touches
				if( key == g_1stTouchKey ) {
					g_1stTouch = g_2ndTouch;
					g_1stTouchKey = g_2ndTouchKey;
					
					g_2ndTouch = g_3rdTouch;
					g_2ndTouchKey = g_3rdTouchKey;

					g_3rdTouch = NULL;
					g_3rdTouchKey = 0;
				} else if( key == g_2ndTouchKey ) {
					g_2ndTouch = g_3rdTouch;
					g_2ndTouchKey = g_3rdTouchKey;
					
					g_3rdTouch = NULL;
					g_3rdTouchKey = 0;
				} else {
					g_3rdTouch = NULL;
					g_3rdTouchKey = 0;
				}

				if( g_rubberBand.isPulled() ) {
					
					// when selecting a projectile, shoot the selected one and deselect it
					if( g_touchedBullet >= 0 ) {
						std::vector<SSProjectile*>::iterator it=g_projectile.begin();
						it += g_touchedBullet;
					
						(*it)->setVelocity(g_rubberBand.getShotDirection()*0.1f);
						g_touchedBullet = -1;
					} else {
						newProjectile([touch locationInView:view]);
					
						g_touchedBullet = selectBullet([touch locationInView:view]);
					
						std::vector<SSProjectile*>::iterator it=g_projectile.begin();
						it += g_touchedBullet;
					
						if( g_themesIndex == 1 )
							(*it)->setBullet();	// only for star wars theme
						
						(*it)->setInitVelocity(g_rubberBand.getShotDirection()*.1f);
					}
				}
				
				// if the band is being pulled, start plucking
				g_rubberBand.startPlucking();

			} else if( [touch phase] == UITouchPhaseBegan ) {
	
				// update the 1st, 2nd & 3rd touches
				if( g_1stTouch == NULL ) {
					g_1stTouch = touch;
					g_1stTouchKey = key;
				} else if( key != g_1stTouchKey ) {
					if( g_2ndTouch == NULL ) {
						g_2ndTouch = touch;
						g_2ndTouchKey = key;
					} else if( key != g_2ndTouchKey ) {
						g_3rdTouch = touch;
						g_3rdTouchKey = key;						
					}
				}
				
				// if one of projectiles is touched, select it
				g_touchedBullet = selectBullet([touch locationInView:view]);

			} else if( touch == g_3rdTouch && [touch phase] == UITouchPhaseMoved ) {

				// when one of projectiles is selected, move its position
				if( g_touchedBullet >= 0 )
					movingBullet(g_touchedBullet, [touch locationInView:view]);
				
				// check if the moving position crossed over the band (the band is held)
				g_rubberBand.checkPullingString([touch locationInView:view]);
				
				g_rubberBand.pullingString([touch locationInView:view]);
				
			}
			
		}
		
		if( g_1stTouch != NULL && g_2ndTouch != NULL ) {
			g_rubberBand.setPoint([g_1stTouch locationInView:view], [g_2ndTouch locationInView:view]);
				
			if( g_3rdTouch != NULL )
				g_rubberBand.pullingString([g_3rdTouch locationInView:view]);
		} else {
			g_rubberBand.removeString();
		}
			
		
	} else {
		
		if( g_eraseBand )
			g_rubberBand.removeString();
	}

}



//---------------------------------------------------------------------
// name: clearSpaceWhenShake()
// description: Clear the space when shaking
//---------------------------------------------------------------------

void clearSpaceWhenShake(double x, double y, double z)
{
	const float violence = 1.5;
	static bool beenhere;
	BOOL shake = false;
	
	if (beenhere) return;
	
	beenhere = true;
	if (x > violence * 1.5 || x < (-1.5* violence))
		shake = true;
	if (y > violence * 2 || y < (-2 * violence))
		shake = true;
	if (z > violence * 3 || z < (-3 * violence))
		shake = true;
	
	if (shake) {
		for( std::vector<SSProjectile*>::iterator it=g_projectile.begin(); it != g_projectile.end(); it++ )
			delete (*it);
		
		g_projectile.clear();
		
		g_rubberBand.removeString();
	} 
	
	beenhere = false;	
}



//---------------------------------------------------------------------
// name: accelCallback()
// description: Accelerometer callback function
//---------------------------------------------------------------------

void accelCallback(double x, double y, double z, void * data)
{
//	Float32 gravitySlew = .1f;//0.005f;
	
	if( g_enableShakeClear )
		clearSpaceWhenShake(x, y, z);
	
	if( g_enableGravity ) {
		g_gravity.x = (y/15.f);// - g_gravity.x)*gravitySlew + g_gravity.x;
		g_gravity.y = (-x/15.f);// - g_gravity.y)*gravitySlew + g_gravity.y;
		g_gravity.z = 0.f;
	} else {
		g_gravity.x = 0.f;
		g_gravity.y = 0.f;
		g_gravity.z = 0.f;
	}
	
}



//---------------------------------------------------------------------
// name: textureMapping()
// description: Texture mapping function
//---------------------------------------------------------------------

void textureMapping(GLuint texture, const GLfloat *pVertices, const GLfloat *pTexCoords)
{
	glEnable( GL_TEXTURE_2D );
	glBindTexture( GL_TEXTURE_2D, texture );
	
	glVertexPointer(2, GL_FLOAT, 0, pVertices);
	glEnableClientState(GL_VERTEX_ARRAY);
	
	glTexCoordPointer(2, GL_FLOAT, 0, pTexCoords);
	glEnableClientState(GL_TEXTURE_COORD_ARRAY);
	
	glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
	
	glDisableClientState(GL_TEXTURE_COORD_ARRAY);
	
	glDisable( GL_TEXTURE_2D );
	
	
}




@implementation ES1Renderer

// Create an ES 1.1 context
- (id) init
{
	if (self = [super init])
	{
		context = [[EAGLContext alloc] initWithAPI:kEAGLRenderingAPIOpenGLES1];
        
        if (!context || ![EAGLContext setCurrentContext:context])
		{
            [self release];
            return nil;
        }
		
		// Create default framebuffer object. The backing will be allocated for the current layer in -resizeFromLayer
		glGenFramebuffersOES(1, &defaultFramebuffer);
		glGenRenderbuffersOES(1, &colorRenderbuffer);
		glBindFramebufferOES(GL_FRAMEBUFFER_OES, defaultFramebuffer);
		glBindRenderbufferOES(GL_RENDERBUFFER_OES, colorRenderbuffer);
		glFramebufferRenderbufferOES(GL_FRAMEBUFFER_OES, GL_COLOR_ATTACHMENT0_OES, GL_RENDERBUFFER_OES, colorRenderbuffer);
	}
	
	//Setup the STK itself
	Stk::setSampleRate(SRATE);
	Stk::showWarnings(false);
	
	// Setup STK instruments
	audioData.pluck = new Plucked(100.0);
	audioData.reverb = new NRev();
	audioData.eClip = new explosionClip();
	audioData.cClip = new collisionClip();		
	
	//init the audio layer
	assert( MoAudio::init( SRATE, FRAMESIZE, NUMCHANNELS) );
	
	
	// start the audio layer
	assert( MoAudio::start( audioCallback, &audioData) );
	
	
	// init touch callback function
	MoTouch::addCallback(touchCallback, &audioData);
	
	// init accelerometer callback function
	MoAccel::addCallback(accelCallback, NULL);
	
	// init AudioData structure for SSRubberBand
	g_rubberBand.setAudioStruct(&audioData);


    // enable texture mapping
    glEnable( GL_TEXTURE_2D );
    // enable blending
    glEnable( GL_BLEND );
    // blend function
    //glBlendFunc( GL_ONE, GL_SRC_COLOR );
	glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
    
    // generate texture name
    glGenTextures( 20, &g_texture[0] );
	
    // bind the texture of a projectile
    glBindTexture( GL_TEXTURE_2D, g_texture[TEX_PROJECTILE] );
    // setting parameters
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
    // load the texture
    MoGfx::loadTexture( @"tiefighter", @"png" );


	// bind the texture of a rubber band
    glBindTexture( GL_TEXTURE_2D, g_texture[TEX_STRING] );
    // setting parameters
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
    // load the texture
    MoGfx::loadTexture( @"lightsaber", @"png" );
	
	// set the texture name of rubberband
	g_rubberBand.setTextureName(&g_texture[TEX_STRING]);

	// bind the texture of the background image
    glBindTexture( GL_TEXTURE_2D, g_texture[TEX_BACKGROUND] );
    // setting parameters
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
    // load the texture
    MoGfx::loadTexture( @"starwars_intro", @"png" );


	for( int i=0; i < 16; i++ ) {
		
		// bind the texture of explosion images
		glBindTexture( GL_TEXTURE_2D, g_texture[TEX_EXPLOSION+i] );
		// setting parameters
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
		// load the texture
		MoGfx::loadTexture( explosionName[i], @"png" );
		
	}

	
	// bind the texture of a blackhole
    glBindTexture( GL_TEXTURE_2D, g_texture[TEX_BLACKHOLE] );
    // setting parameters
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
    // load the texture
    MoGfx::loadTexture( @"turret", @"png" );

	
	return self;
}

- (void) render
{

	static const GLfloat bgVertices[] = {
        -1.0f,  -0.6f,
		1.0f,  -0.6f,
        -1.0f,   0.6f,
		1.0f,   0.6f
    };
	
	static const GLfloat texCoordsBg[] = {
		0, 0.5f,
		1, 0.5f,
		0, 0,
		1, 0
	};
		
	static const GLfloat blackholeVertices[] = {
        -0.15f,  -0.15f,
		0.15f,  -0.15f,
        -0.15f,   0.15f,
		0.15f,   0.15f,
    };

	static const GLfloat texCoordsTurret0[] = {
		0, 1,
		1, 1,
		0, 0,
		1, 0,
	};
	
	static const GLfloat texCoordsTurret1[] = {
		0, 0,
		1, 0,
		0, 1,
		1, 1,
	};

	
	
	// This application only creates a single context which is already set current at this point.
	// This call is redundant, but needed if dealing with multiple contexts.
    [EAGLContext setCurrentContext:context];
    
	// This application only creates a single default framebuffer which is already bound at this point.
	// This call is redundant, but needed if dealing with multiple framebuffers.
    glBindFramebufferOES(GL_FRAMEBUFFER_OES, defaultFramebuffer);
    glViewport(0, 0, backingWidth, backingHeight);
    
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	
	glOrthof(-1.0f, 1.0f, -1.5f, 1.5f, -1.0f, 1.0f);  ///// mapping projection before
	glRotatef( 90, 0, 0, 1);
	
    glClearColor(0.f, 0.f, 0.f, 1.0f);
    glClear(GL_COLOR_BUFFER_BIT);

	glEnableClientState(GL_VERTEX_ARRAY);
	glDisable( GL_TEXTURE_2D );

	
	// draw background
	
	if( g_themesIndex == 1 ) { 		// star wars theme
		
		textureMapping(g_texture[TEX_BACKGROUND], bgVertices, texCoordsBg);
		
		// draw turret
	
		glPushMatrix();
	
			glTranslatef(1.35f, -0.85f, 0);
	
			textureMapping(g_texture[TEX_BLACKHOLE], blackholeVertices, texCoordsTurret0);

		glPopMatrix();

		glPushMatrix();

			glTranslatef(1.35f, 0.85f, 0);
	
			textureMapping(g_texture[TEX_BLACKHOLE], blackholeVertices, texCoordsTurret1);

		glPopMatrix();
	} else {		// no texture

		CGPoint origin={0.f, 0.f};
		
		// draw blackhole
		
		glLineWidth(3);
		
		glPushMatrix();
		
			glTranslatef(1.39f, 0.88f, 0);
			glRotatef(-45, 0, 0, 1);
			glColor4f(.25f, .25f, .25f, 1.f);
			drawEllipse (10, 0.08f, 0.02f, origin, false);

		glPopMatrix();		
		
		glPushMatrix();
		
			glTranslatef(1.36f, 0.86f, 0);
			glRotatef(-45, 0, 0, 1);
			glColor4f(.5f, .5f, .5f, 1.f);
			drawEllipse (10, 0.12f, 0.03f, origin, false);
		
		glPopMatrix();		
		
		glPushMatrix();

			glTranslatef(1.33f, 0.84f, 0);
			glRotatef(-45, 0, 0, 1);
			glColor4f(.75f, .75f, .75f, 1.f);
			drawEllipse (10, 0.16f, 0.04f, origin, false);
	
		glPopMatrix();
		
		glPushMatrix();

			glTranslatef(1.3f, 0.82f, 0);
			glRotatef(-45, 0, 0, 1);
			glColor4f(1.f, 1.f, 1.f, 1.f);
			drawEllipse (10, 0.2f, 0.05f, origin, false);
		
		glPopMatrix();

		glPushMatrix();
		
			glTranslatef(1.39f, -0.88f, 0);
			glRotatef(45, 0, 0, 1);
			glColor4f(.25f, .25f, .25f, 1.f);
			drawEllipse (10, 0.08f, 0.02f, origin, false);
		
		glPopMatrix();		
		
		glPushMatrix();
		
			glTranslatef(1.36f, -0.86f, 0);
			glRotatef(45, 0, 0, 1);
			glColor4f(.5f, .5f, .5f, 1.f);
			drawEllipse (10, 0.12f, 0.03f, origin, false);
		
		glPopMatrix();		
		
		glPushMatrix();
		
			glTranslatef(1.33f, -0.84f, 0);
			glRotatef(45, 0, 0, 1);
			glColor4f(.75f, .75f, .75f, 1.f);
			drawEllipse (10, 0.16f, 0.04f, origin, false);
		
		glPopMatrix();
		
		glPushMatrix();
		
			glTranslatef(1.3f, -0.82f, 0);
			glRotatef(45, 0, 0, 1);
			glColor4f(1.f, 1.f, 1.f, 1.f);
			drawEllipse (10, 0.2f, 0.05f, origin, false);
		
		glPopMatrix();
		
	}

	// draw rubber band
	g_rubberBand.draw();
	
	// test collision
	collisionProc(&audioData);


	// draw projectiles
	for( std::vector<SSProjectile*>::iterator it = g_projectile.begin(); it != g_projectile.end(); it++ ){
		if( (*it)->draw(g_themesIndex, &g_texture[0], &audioData) == DELETE_PROJECTILE ) {
			delete (*it);
			g_projectile.erase(it);
			it--;
		}
	}
	
	// This application only creates a single color renderbuffer which is already bound at this point.
	// This call is redundant, but needed if dealing with multiple renderbuffers.
    glBindRenderbufferOES(GL_RENDERBUFFER_OES, colorRenderbuffer);
    [context presentRenderbuffer:GL_RENDERBUFFER_OES];
}


- (void) readPreference
{
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	if( [defaults objectForKey: @"Gravity"] != nil )
		g_enableGravity = [defaults boolForKey: @"Gravity"];
	else
		g_enableGravity = true;
	
	if( [defaults objectForKey: @"Shake"] != nil )
		g_enableShakeClear = [defaults boolForKey: @"Shake"];
	else
		g_enableShakeClear = true;	
	
	if( [defaults objectForKey: @"Tension"] != nil )
		g_tension = [ defaults floatForKey:@"Tension"];
	else
		g_tension = 0.8f;
	
	if( [defaults objectForKey: @"WallResist"] != nil )
		g_wallResist = [ defaults floatForKey:@"WallResist"];
	else
		g_wallResist = 0.95f;
	
	if( [defaults objectForKey: @"AirResist"] != nil )
		g_airResist = [ defaults floatForKey:@"AirResist"];
	else
		g_airResist = 0.99f;
	
	if( [defaults objectForKey: @"ReverbTime"] != nil )
		g_reverbTime = [ defaults floatForKey:@"ReverbTime"];
	else
		g_reverbTime = 0.f;
	
	audioData.reverb->setT60(g_reverbTime);
	
	if( [defaults objectForKey: @"EraseBand"] != nil )
		g_eraseBand = [defaults boolForKey: @"EraseBand"];
	else
		g_eraseBand = true;	
	
	if( [defaults objectForKey: @"Themes"] != nil )
		g_themesIndex = [defaults integerForKey: @"Themes"];
	else
		g_themesIndex = 0;
	
	g_rubberBand.setTheme(g_themesIndex);

	g_rubberBand.setStringTension(g_tension);
	
}


- (BOOL) resizeFromLayer:(CAEAGLLayer *)layer
{	
	// Allocate color buffer backing based on the current layer size
    glBindRenderbufferOES(GL_RENDERBUFFER_OES, colorRenderbuffer);
    [context renderbufferStorage:GL_RENDERBUFFER_OES fromDrawable:layer];
	glGetRenderbufferParameterivOES(GL_RENDERBUFFER_OES, GL_RENDERBUFFER_WIDTH_OES, &backingWidth);
    glGetRenderbufferParameterivOES(GL_RENDERBUFFER_OES, GL_RENDERBUFFER_HEIGHT_OES, &backingHeight);
	
    if (glCheckFramebufferStatusOES(GL_FRAMEBUFFER_OES) != GL_FRAMEBUFFER_COMPLETE_OES)
	{
		NSLog(@"Failed to make complete framebuffer object %x", glCheckFramebufferStatusOES(GL_FRAMEBUFFER_OES));
        return NO;
    }
    
    return YES;
}

- (void) dealloc
{
	// Tear down GL
	if (defaultFramebuffer)
	{
		glDeleteFramebuffersOES(1, &defaultFramebuffer);
		defaultFramebuffer = 0;
	}

	if (colorRenderbuffer)
	{
		glDeleteRenderbuffersOES(1, &colorRenderbuffer);
		colorRenderbuffer = 0;
	}
	
	// Tear down context
	if ([EAGLContext currentContext] == context)
        [EAGLContext setCurrentContext:nil];
	
	[context release];
	context = nil;
	
	[super dealloc];
}

@end
