//
//  FlipsideViewController.m
//  hw2
//
//  Created by slork on 1/18/10.
//  Copyright Stanford University 2010. All rights reserved.
//

#import "FlipsideViewController.h"


@implementation FlipsideViewController

@synthesize delegate;
@synthesize gravitySW;
@synthesize shakeSW;
@synthesize eraseBandSW;
@synthesize tensionSlider;
@synthesize wallResistSlider;
@synthesize airResistSlider;
@synthesize revTimeSlider;
@synthesize revTimeLabel;
@synthesize themesSeg;


- (void)viewDidLoad {
    [super viewDidLoad];
    self.view.backgroundColor = [UIColor viewFlipsideBackgroundColor];
	    
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	if( [defaults objectForKey: @"Gravity"] != nil )
		gravity = [defaults boolForKey: @"Gravity"];
	else
		gravity = true;

	if( [defaults objectForKey: @"Shake"] != nil )
		shake = [defaults boolForKey: @"Shake"];
	else
		shake = true;
	
	if( [defaults objectForKey: @"Tension"] != nil )
		tension = [defaults floatForKey: @"Tension"];
	else
		tension = 0.8f;
	
	if( [defaults objectForKey: @"WallResist"] != nil )
		wallResist = [defaults floatForKey: @"WallResist"];
	else
		wallResist = 0.95f;

	if( [defaults objectForKey: @"AirResist"] != nil )
		airResist = [defaults floatForKey: @"AirResist"];
	else
		airResist = 0.98f;
	
	if( [defaults objectForKey: @"EraseBand"] != nil )
		eraseBand = [defaults boolForKey: @"EraseBand"];
	else
		eraseBand = true;

	if( [defaults objectForKey: @"ReverbTime"] != nil )
		revTime = [defaults floatForKey: @"ReverbTime"];
	else
		revTime = 0.f;

	if( [defaults objectForKey: @"Themes"] != nil )
		themesIndex = [defaults integerForKey: @"Themes"];
	else
		themesIndex = 0;
	
	
	gravitySW.on = gravity;
	shakeSW.on = shake;
	
	tensionSlider.value = tension;
	wallResistSlider.value = wallResist;
	airResistSlider.value = airResist;
	
	revTimeSlider.value = revTime;
	revTimeLabel.text = [NSString stringWithFormat: @"%.2f sec", revTime];
	
	eraseBandSW.on = eraseBand;
	
	themesSeg.selectedSegmentIndex = themesIndex;
}


- (IBAction)done {
	[self.delegate flipsideViewControllerDidFinish:self];

}

- (IBAction)changeGravitySW {
	gravity = gravitySW.on;
}

- (IBAction)changeShakeSW {
	shake = shakeSW.on;
}


- (IBAction)changeTensionSlider:(id)sender {
	tension = ((UISlider *)sender).value;
}


- (IBAction)changeWallResistSlider:(id)sender {
	wallResist = ((UISlider *)sender).value;
}


- (IBAction)changeAirResistSlider:(id)sender {
	airResist = ((UISlider *)sender).value;
}


- (IBAction)changeRevTimeSlider:(id)sender {
	revTime = ((UISlider *)sender).value;
	
	revTimeLabel.text = [NSString stringWithFormat: @"%.2f sec", revTime];
}



- (IBAction)changeEraseBandSW {
	eraseBand = eraseBandSW.on;
}


- (IBAction)changeThemes {
	themesIndex = themesSeg.selectedSegmentIndex;
}


/*
 // Override to allow orientations other than the default portrait orientation.
 - (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
 // Return YES for supported orientations
 return (interfaceOrientation == UIInterfaceOrientationPortrait);
 }
 */


- (void) viewWillDisappear: (BOOL) animated
{
	
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

	[defaults setBool:gravity forKey:@"Gravity"];
	[defaults setBool:shake forKey:@"Shake"];

	[defaults setFloat:tension forKey:@"Tension"];
	[defaults setFloat:wallResist forKey:@"WallResist"];
	[defaults setFloat:airResist forKey:@"AirResist"];

	[defaults setFloat:revTime forKey:@"ReverbTime"];

	[defaults setBool:eraseBand forKey:@"EraseBand"];

	[defaults setInteger:themesIndex forKey:@"Themes"];
	
	[super viewWillDisappear: animated];
}




- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
	
	// Release any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
	// Release any retained subviews of the main view.
	// e.g. self.myOutlet = nil;
}


- (void)dealloc {
    [super dealloc];
}


@end
