/*
 *  SSObject.h
 *  SlingShot
 *
 *  Created by Keun Sup Lee on 2/6/10.
 *  Copyright 2010 __MyCompanyName__. All rights reserved.
 *
 */
#import "mo_audio.h"
#import "mo_gfx.h"
#import "ES1Renderer.h"


#define NUM_STRING_SAMPLE	200
#define SPEED_OF_SOUND		340		// m/sec


#define X_RESOL				480
#define Y_RESOL				320

#define XAXIS_MAPPING(X)	(((X_RESOL/2) - (X))/(X_RESOL/3))
#define YAXIS_MAPPING(Y)	(((Y_RESOL/2) - (Y))/(Y_RESOL/2))

#define PROJECTILE_RADIUS	0.1f

#define DELETE_PROJECTILE	0
#define NOP					1

#define TEX_PROJECTILE		0
#define TEX_STRING			1
#define TEX_BACKGROUND		2
#define TEX_EXPLOSION		3
#define TEX_BLACKHOLE		19



class SSProjectile {
public:
	SSProjectile();
	SSProjectile(Vector3D position);
	
	Vector3D getPoint() { return pos; }
	void setPoint(CGPoint point);
	void setPoint(Vector3D point);
	void setInitVelocity(Vector3D velocity);
	void setVelocity(CGPoint velocity);
	void setVelocity(Vector3D velocity) { vel = velocity; }
	Vector3D getVelocity() { return vel; }
	void stopProjectile() { vel.setAll(0); }
	void setBullet() { bullet = true; }
	bool getBullet() { return bullet; }
	void setExplosion() { explosionOn = true; }
	void setCollision() { collisionOn = true; }
	void removeProjectile() { justRemove = true; }
	
	void slowDown(CGPoint slowFactor);
	void slowDown(GLfloat x, GLfloat y, GLfloat z);
	
	int draw(int themeType, GLuint *tex, struct AudioData *audioData);
	
protected:
	Vector3D pos;	// position
	Vector3D vel;	// velocity
	bool bullet;
	
	bool justRemove;
	bool collisionOn;
	int collisionCount;
	bool explosionOn;
	int explosionFrame;
	int explosionSlow;
	
};


class SSRubberBand {
public:
	SSRubberBand();
	
	void setPoint(CGPoint point0, CGPoint point1);
	void pullingString(CGPoint point);
	void setTheme(int themeType) { theme = themeType; }
	void setTextureName(GLuint *tex) { texture = tex; }
	
	bool checkPullingString(CGPoint point);
	void updateStringState();
	void removeString();
	
	void startPlucking();
	
	void setStringTension(GLfloat t) { tension = t; }
	void setAudioStruct(struct AudioData *data) { audioData = data; }
	
	Vector3D getShotDirection() { return shotDirection; }
	bool isPulled() { return pulling; }

	void draw();
	
protected:
	SAMPLE vertices[2*NUM_STRING_SAMPLE];
	SAMPLE delayBuf0[NUM_STRING_SAMPLE];
	SAMPLE delayBuf1[NUM_STRING_SAMPLE];
	
	Vector3D startPoint;
	Vector3D endPoint;
	Vector3D pullingPoint;
	Vector3D shotDirection;
	
	GLfloat stringLength;
	GLfloat theta;
	GLfloat stringMagnitude;
	GLfloat tension;
	
	int theme;
	GLuint *texture;
	
	bool drawString;
	bool pulling;
	bool plucking;
	
	struct AudioData *audioData;

	
};
